//-------------------------------------------------------------------
// WatchdogTimerTestDlg.cpp
//
// Demonstrates use of the StrongARM watchdog timer.
//
// Users can set and change the timeout values for both the watchdog 
// timer and for the application timer that resets the watchdog.
//
// Once the watchdog timer has been enabled, the user can change
// the settings or exit from the application, which will cause
// the watchdog to trip and reset the system.
//
//
// (c)2002 Applied Data Systems
//
// 4dec02 cl
//  - Initial release
// 17dec02 ak
//  - add header and comment
//  - update user messages and variable names
//  - allow timer to get reinitialized repeatedly
//-------------------------------------------------------------------

#include "stdafx.h"
#include "WatchdogTimerTest.h"
#include "WatchdogTimerTestDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWatchdogTimerTestDlg dialog

//-------------------------------------------------------------------
//-------------------------------------------------------------------
CWatchdogTimerTestDlg::CWatchdogTimerTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CWatchdogTimerTestDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CWatchdogTimerTestDlg)
		// NOTE: the ClassWizard will add member initialization here			m_wd_enable_time = _T("");
			m_wd_enable_time = _T("");
			m_wd_update_time = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void CWatchdogTimerTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWatchdogTimerTestDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
		DDX_Text(pDX, IDC_WD_ENABLE_TIME, m_wd_enable_time);
		DDX_Text(pDX, IDC_WD_UPDATE_TIME, m_wd_update_time);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CWatchdogTimerTestDlg, CDialog)
	//{{AFX_MSG_MAP(CWatchdogTimerTestDlg)
	ON_BN_CLICKED(IDC_EXEC, OnExec)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWatchdogTimerTestDlg message handlers

//-------------------------------------------------------------------
//-------------------------------------------------------------------
BOOL CWatchdogTimerTestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	CenterWindow(GetDesktopWindow());	// center to the hpc screen

	return TRUE;  // return TRUE  unless you set the focus to a control
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
#include "SA1110.hpp"
#include "MapIO.hpp"
#include "WDTimer.hpp"

#define EVENT_DISPLAY_SECOND 0xf55f

int		secondCounter = 0;
int		updateCounter = 0;
ULONG	watchdogEnablePeriod=10;
ULONG	watchdogUpdateTime=4;

BOOL	WatchdogInitialized = FALSE;
UINT	UpdateTimer;

//-------------------------------------------------------------------
// Initializes the watchdog timer and application timers.
//-------------------------------------------------------------------
void CWatchdogTimerTestDlg::OnExec() 
{
	watchdogEnablePeriod = GetDlgItemInt(IDC_WD_ENABLE_TIME);
	watchdogUpdateTime = GetDlgItemInt(IDC_WD_UPDATE_TIME);
	if (watchdogUpdateTime>watchdogEnablePeriod) {
		MessageBox(TEXT("WDTimerEnable value must be greater than WDTimerUpdate"), TEXT("ERROR"), MB_ICONSTOP);
		return;
	}
	OstRegPtr = (SA1100_OST *)MapPhysicalAddr((void *)OST_PHYSICAL_BASE,0x1000);	// 1 Page
	if(OstRegPtr == NULL){
		MessageBox(TEXT("ERROR OstRegPtr == NULL"), TEXT("ERROR"), MB_ICONSTOP);
		return;
	}
	//Enable the Watchdog Timer
	WDTimerEnable(watchdogEnablePeriod);

	if (WatchdogInitialized)
		KillTimer (UpdateTimer); // timer has already been initialized; kill it first
	else
		//display second counter every 1000 milli-second (init this timer once)
		SetTimer(EVENT_DISPLAY_SECOND, 1000, 0);

	//call the WDTimerUpdate() counter every <watchdogUpdateTime> second
	UpdateTimer = SetTimer(watchdogUpdateTime, watchdogUpdateTime*1000, 0);

	WatchdogInitialized = TRUE;
}

//-------------------------------------------------------------------
// Updates the display and re-arms the watchdog timer.
// Tracks the number of times the watchdog timer has been re-armed.
//-------------------------------------------------------------------
void CWatchdogTimerTestDlg::OnTimer(UINT nIDEvent) 
{
	//Display second counter evernt
	if (nIDEvent == EVENT_DISPLAY_SECOND) {
		secondCounter++;
		SetDlgItemInt(IDC_SECOND_COUNTER, secondCounter);
	}
	//Update the Watchdog Timer Event
	if (nIDEvent == watchdogUpdateTime) {
		//Update the Watchdog Timer
//		WDTimerUpdate(watchdogUpdateTime); 
		WDTimerReset(); 
		updateCounter++;
		SetDlgItemInt(IDC_UPDATE_COUNTER, updateCounter);
	}
	CDialog::OnTimer(nIDEvent);
}

//-------------------------------------------------------------------
// Presents an information/warning message when the Exit button 
// is pressed.
//-------------------------------------------------------------------
void CWatchdogTimerTestDlg::OnOK() 
{
	TCHAR debugMsg[256];
	
	wsprintf(debugMsg, TEXT("The Watchdog Timer is currently enabled.\r\n"));
	wsprintf(debugMsg, TEXT("%sSystem will reset within %d seconds of exiting this app.\r\n\n"), debugMsg, watchdogEnablePeriod);
	wsprintf(debugMsg, TEXT("%sDo you want to exit program now?\r\n"), debugMsg);
	if (MessageBox(debugMsg, TEXT("Question"), MB_ICONQUESTION | MB_YESNO) == IDYES)
		CDialog::OnOK();
}
// end of sample code
