/*++
Copyright (c) 1998~ 2002 Applied Data Systems Inc.
			www.applieddata.net

Module:

    sa1110.h

Module Description:

    This file contains definitions of the SA-1110 on-chip hardware registers.
    The structure members corresponding to on-chip registers should be
    accessed by using the register access routines and not dereferenced
    manually.  That is to say, use

      foo = READ_REGISTER_ULONG(&UartPtr->UTCR0);

    instead of 

      foo = UartPtr->UTCR0;

    to read UTCR0.

Revision History:

--*/

#ifndef _SA1110_H_
#define _SA1110_H_

//  Register Physical Addresses
//------------------------------
#define OST_PHYSICAL_BASE          0x90000000  // OS Timer


//OS Timer (0x90000000)
//---------------------

typedef struct _SA1100_OST
{
    ULONG  OSMR0;          // [0x00] OS Timer Match Register 0
    ULONG  OSMR1;          // [0x04] OS Timer Match Register 1
    ULONG  OSMR2;          // [0x08] OS Timer Match Register 2
    ULONG  OSMR3;          // [0x0C] OS Timer Match Register 3
    ULONG  OSCR;           // [0x10] OS Timer Count Register
    ULONG  OSSR;           // [0x14] OS Timer Status Register
    ULONG  OWER;           // [0x18] OS Timer Watchdog Enable Register
    ULONG  OIER;           // [0x1C] OS Timer Interrupt Enable Register
} SA1100_OST;


//  Bit Defines and Common Settings for Registers
//-----------------------------------------------

// Operating System Timer
//-----------------------

// OS Timer Watchdog Match Enable Register (OWER)
#define OWER_WME    (1 << 0)  // Watchdog match enable

// OS Timer Status Register (OSSR)
#define OSSR_M0     (1 << 0)  // Match status channel 0
#define OSSR_M1     (1 << 1)  // Match status channel 1
#define OSSR_M2     (1 << 2)  // Match status channel 2
#define OSSR_M3     (1 << 3)  // Match status channel 3

// OS Timer Interrupt Enable Register (OIER)
#define OIER_E0     (1 << 0)  // Interrupt enable channel 0
#define OIER_E1     (1 << 1)  // Interrupt enable channel 1
#define OIER_E2     (1 << 2)  // Interrupt enable channel 2
#define OIER_E3     (1 << 3)  // Interrupt enable channel 3


#define INPUT_CLOCK 	3686400		// 3.6864 MHz
#define HNS_TO_TICKS(HNS)      ((ULONG)(((ULONGLONG)(HNS)*1583296744) >> 32))

ULONG __inline READ_REGISTER_ULONG(PULONG Register)
{
    return *(volatile unsigned long * const)Register;
}

VOID __inline WRITE_REGISTER_ULONG(PULONG Register, ULONG value)
{
    *(volatile unsigned long * const)Register = value;
}

#endif // _SA1110_H_
